@@ -48,6 +48,12 @@ sub jordan_totient {
   _validate_positive_integer($n);
   return Math::Prime::Util::PP::jordan_totient($k, $n);
 }
+sub ramanujan_sum {
+  my($k,$n) = @_;
+  _validate_positive_integer($k);
+  _validate_positive_integer($n);
+  return Math::Prime::Util::PP::ramanujan_sum($k, $n);
+}
 sub carmichael_lambda {
   my($n) = @_;
   _validate_positive_integer($n);
@@ -194,6 +200,12 @@ sub is_strong_pseudoprime {
   croak "No bases given to is_strong_pseudoprime" unless @bases;
   return Math::Prime::Util::PP::is_strong_pseudoprime($n, @bases);
 }
+sub is_euler_plumb_pseudoprime {
+  my($n) = @_;
+  return 0 if defined $n && int($n) < 0;
+  _validate_positive_integer($n);
+  return Math::Prime::Util::PP::is_euler_plumb_pseudoprime($n);
+}
 sub is_lucas_pseudoprime {
   my($n) = @_;
   return 0 if defined $n && int($n) < 0;
@@ -221,10 +233,12 @@ sub is_almost_extra_strong_lucas_pseudoprime {
   return Math::Prime::Util::PP::is_almost_extra_strong_lucas_pseudoprime($n, $increment);
 }
 sub is_perrin_pseudoprime {
-  my($n) = @_;
+  my($n,$restrict) = @_;
   return 0 if defined $n && int($n) < 0;
+  $restrict = 0 unless defined $restrict;
   _validate_positive_integer($n);
-  return Math::Prime::Util::PP::is_perrin_pseudoprime($n);
+  _validate_positive_integer($restrict);
+  return Math::Prime::Util::PP::is_perrin_pseudoprime($n, $restrict);
 }
 sub is_catalan_pseudoprime {
   my($n) = @_;
@@ -286,6 +300,7 @@ sub is_quasi_carmichael {
 sub is_primitive_root {
   my($a,$n) = @_;
   $n = -$n if defined $n && $n < 0;
+  $a %= $n if defined $a && $a < 0;
   _validate_positive_integer($a);
   _validate_positive_integer($n);
   return Math::Prime::Util::PP::is_primitive_root($a,$n);
@@ -372,7 +387,7 @@ sub znlog {
 
 sub znprimroot {
   my($n) = @_;
-  $n = -$n if defined $n && $n =~ /^-\d+/;   # TODO: fix this for string bigints
+  $n =~ s/^-(\d+)/$1/ if defined $n;
   _validate_positive_integer($n);
   return Math::Prime::Util::PP::znprimroot($n);
 }
@@ -536,6 +551,18 @@ sub sqrtint {
   _validate_integer($n);
   return Math::Prime::Util::PP::sqrtint($n);
 }
+sub rootint {
+  my($n, $k, $refp) = @_;
+  _validate_positive_integer($n);
+  _validate_positive_integer($k);
+  return Math::Prime::Util::PP::rootint($n, $k, $refp);
+}
+sub logint {
+  my($n, $b, $refp) = @_;
+  _validate_positive_integer($n);
+  _validate_positive_integer($b);
+  return Math::Prime::Util::PP::logint($n, $b, $refp);
+}
 
 sub legendre_phi {
   my($x, $a) = @_;
@@ -568,6 +595,13 @@ sub is_power {
   $vn = '-'.$vn if $n < 0;
   return Math::Prime::Util::PP::is_power($vn, $a, $refp);
 }
+sub is_prime_power {
+  my($n, $refp) = @_;
+  my $vn = "$n";  $vn =~ s/^-//;
+  _validate_positive_integer($vn);
+  $vn = '-'.$vn if $n < 0;
+  return Math::Prime::Util::PP::is_prime_power($vn, $refp);
+}
 sub valuation {
   my($n, $k) = @_;
   $n = -$n if defined $n && $n < 0;
