@@ -4,7 +4,7 @@ use warnings;
 
 BEGIN {
   $ntheory::AUTHORITY = 'cpan:DANAJ';
-  $ntheory::VERSION = '0.58';
+  $ntheory::VERSION = '0.59';
 }
 
 BEGIN {
@@ -51,6 +51,7 @@ See L<Math::Prime::Util> for complete documentation.
 
   is_pseudoprime(n,bases)                  Fermat probable prime test
   is_euler_pseudoprime(n,bases)            Euler test to bases
+  is_euler_plumb_pseudoprime(n)            Euler Criterion test
   is_strong_pseudoprime(n,bases)           Miller-Rabin test to bases
   is_lucas_pseudoprime(n)                  Lucas test
   is_strong_lucas_pseudoprime(n)           strong Lucas test
@@ -59,7 +60,7 @@ See L<Math::Prime::Util> for complete documentation.
   is_frobenius_pseudoprime(n, [a,b])       Frobenius quadratic test
   is_frobenius_underwood_pseudoprime(n)    combined PSP and Lucas
   is_frobenius_khashin_pseudoprime(n)      Khashin's 2013 Frobenius test
-  is_perrin_pseudoprime(n)                 Perrin test
+  is_perrin_pseudoprime(n [,r])            Perrin test
   is_catalan_pseudoprime(n)                Catalan test
   is_bpsw_prime(n)                         combined SPSP-2 and ES Lucas
   miller_rabin_random(n, ntests)           perform random-base MR tests
@@ -153,14 +154,20 @@ See L<Math::Prime::Util> for complete documentation.
   fromdigits(\@d,[,base])             convert base digit vector to number
   fromdigits(str,[,base])             convert base digit string to number
   sumdigits(n)                        sum of digits, with optional base
-  is_power(n)                         return k if n = p^k for integer p, max k
-  is_power(n,k)                       return 1 if n = p^k for integer p and k
-  is_power(n,k,\$root)                as above but set root to p.
+  is_power(n)                         return k if n = c^k for integer c
+  is_power(n,k)                       return 1 if n = c^k for integer c, k
+  is_power(n,k,\$root)                as above but also set $root to c.
+  is_prime_power(n)                   return k if n = p^k for prime p
+  is_prime_power(n,\$p)               as above but also set $p to p
   is_square_free(n)                   return true if no repeated factors
   is_carmichael(n)                    is n a Carmichael number
   is_quasi_carmichael(n)              is n a quasi-Carmichael number
   is_primitive_root(r,n)              is r a primitive root mod n
   sqrtint(n)                          integer square root
+  rootint(n,k)                        integer k-th root
+  rootint(n,k,\$rk)                   as above but also set $rk to r^k
+  logint(n,b)                         integer logarithm
+  logint(n,b,\$be)                    as above but also set $be to b^e.
   gcd(@list)                          greatest common divisor
   lcm(@list)                          least common multiple
   gcdext(x,y)                         return (u,v,d) where u*x+v*y=d
@@ -186,6 +193,7 @@ See L<Math::Prime::Util> for complete documentation.
   euler_phi(beg, end)                 Euler totient for a range
   jordan_totient(n,k)                 Jordan's totient
   carmichael_lambda(n)                Carmichael's Lambda function
+  ramanujan_sum(k,n)                  Ramanujan's sum
   exp_mangoldt                        exponential of Mangoldt function
   liouville(n)                        Liouville function
   znorder(a,n)                        multiplicative order of a mod n
